@@ -133,7 +133,9 @@ class GitLayerIndex(object):
         self.readfile = SandboxReader(self.path).readfile
         self._files = {}            # dict of GitLayerFile instanced indexed by URI
         self._modtime_cache = {}    # dict of os.stat.mtime() indexed by full path
-        self.scan_files(initial=True)
+        self.initial_scan = True
+        self.scan_files()
+        self.initial_scan = False
 
     def _get_mime_type(self, ext):
         '''
@@ -146,7 +148,7 @@ class GitLayerIndex(object):
                 return mime
         return self.DEFAULT_MIME
 
-    def scan_files(self, initial=False):
+    def scan_files(self):
         '''
             os.walk()'s over the web root and checks each files mtime to see if it
             needs reloading. Files are lazily reloaded when their mtime changes
@@ -165,33 +167,57 @@ class GitLayerIndex(object):
                 modtime = os.path.getmtime(full_path)
                 if modtime > self._modtime_cache.get(full_path, 0):
                     files_updated = True
-                    uri = full_path[len(self.path):]
-                    uri = '/'.join(uri.split(os.sep))
-                    if not initial:
-                        log.info('mtime changed, updating: {}'.format(uri))
-                    self._modtime_cache[full_path] = modtime
-                    self._files[uri] = self._store_file(full_path, root,
-                        file_name, modtime, uri)
-                    if self._files[uri].is_remap:
-                        # file has been remapped, change its URI
-                        remapped_uri = self._files[uri].remapped_uri
-                        if remapped_uri not in self._files:
-                            # only change the URI if another file hasn't used it
-                            log.info('remapped {} -> {}'.format(uri, remapped_uri))
-                            self._files[remapped_uri] = copy(self._files[uri])
-                            self._files[remapped_uri].uri = remapped_uri
-                            del self._files[uri]
-                            uri = remapped_uri
-                        else:
-                            log.error('failed to remap {} -> {}, uri already in use'
-                                .format(uri, remapped_uri))
-                    if uri in self.INDEX_FILES:
-                        # store the same file again under the root uri
-                        self._files['/'] = copy(self._files[uri])
-                        self._files['/'].is_alias = True
+                    self._scan_file(modtime, root, file_name, full_path)
         if files_updated:
+            for uri,f in self._files.iteritems():
+                if f.parent_uri:
+                    log.info('detected parent: {} for file: {}'.format(f.parent_uri, f.uri))
+                    f.parent = self.get_file(f.parent_uri)
             self.generate_sitemap()
 
+    def _scan_file(self, modtime, root, file_name, full_path):
+        uri = full_path[len(self.path):]
+        uri = '/'.join(uri.split(os.sep))
+        if not self.initial_scan:
+            log.info('mtime changed, updating: {}'.format(uri))
+        self._modtime_cache[full_path] = modtime
+        self._files[uri] = self._store_file(full_path, root, file_name, modtime, uri)
+        if self._files[uri].is_base and not self.initial_scan:
+            self._handle_template_file(self._files[uri])
+            # no further processing as a full rescap is being performed
+            return
+        elif self._files[uri].is_remap:
+            self._handle_remap_file(self._files[uri])
+        if uri in self.INDEX_FILES:
+            # store the same file again under the root uri
+            self._files['/'] = copy(self._files[uri])
+            self._files['/'].is_alias = True
+
+    def _handle_template_file(self, f):
+        log.info('changed file is a base template, reloading cache')
+        self.initial_scan = True
+        # clear modtimes for all files that are not this file
+        self._modtime_cache = {}
+        self._modtime_cache[self.full_path(f)] = f._modtime
+        self._files = {}
+        self._files[f.uri] = f
+        # force a rescan of all files
+        self.scan_files()
+        self.initial_scan = False
+
+    def _handle_remap_file(self, f):
+        # file has been remapped, change its URI
+        remapped_uri = f.remapped_uri
+        if remapped_uri not in self._files or f.original_uri == f.uri:
+            # only change the URI if another file hasn't used it
+            log.info('remapped {} -> {}'.format(f.uri, remapped_uri))
+            self._files[remapped_uri] = copy(self._files[f.uri])
+            self._files[remapped_uri].uri = remapped_uri
+            del self._files[f.uri]
+        else:
+            log.error('failed to remap {} -> {}, uri already in use'
+                .format(f.uri, remapped_uri))
+
     def _store_file(self, full_path, root, file_name, modtime, uri):
         '''
             Create and store a GitLayerFile instance.
@@ -225,6 +251,7 @@ class GitLayerIndex(object):
         f.is_error401 = uri == self.ERROR_401_FILE
         f.is_error404 = uri == self.ERROR_404_FILE
         f.is_base = name.startswith('_')
+        f._modtime = modtime
         f.is_html = f.do_jinja and not f.is_error401 and not f.is_error404 \
             and not f.is_base
         if (f.do_markdown or f.do_jinja) and f.large:
@@ -291,9 +318,11 @@ class GitLayerIndex(object):
             if env.template_locale:
                 f.locale = env.template_locale
                 f.locale_changed = True
-            if env.template_uri and env.template_uri:
+            if env.template_uri:
                 f.remapped_uri = env.template_uri
                 f.is_remap = True
+            if env.template_parenturi:
+                f.parent_uri = env.template_parenturi
             for concat_uri, concat_files in env.template_concat.items():
                 if concat_uri not in self._files:
                     self._concat_file(concat_uri, concat_files)
@@ -333,6 +362,7 @@ class GitLayerIndex(object):
         md = MarkdownRenderer(template_data, silent=True)
         header, body = md.header, md.body
         template = u''
+        contentblock = 'content'
         template_uri = None
         template_title = None
         template_date = None
@@ -357,13 +387,17 @@ class GitLayerIndex(object):
                     template += u'{{% locale \'{}\' %}}\n'.format(v)
                 elif k == 'timezone':
                     template += u'{{% timezone \'{}\' %}}\n'.format(v)
+                elif k == 'parent':
+                    template += u'{{% parent \'{}\' %}}\n'.format(v)
                 elif k == 'tags':
                     template += u'{{% tags %}}{}{{% endtags %}}\n'.format(v)
                 elif k == 'groups':
-                    template += u'{{% groups %}}{}{{% groups %}}\n'.format(v)
+                    template += u'{{% groups %}}{}{{% endgroups %}}\n'.format(v)
+                elif k == 'contentblock':
+                    contentblock = str(v)
                 elif len(b) > 1 and b[0] == 'block':
                     template += u'{{ % block {} %}}\n{}\n{{% endblock %}}\n'.format(b[1], v)
-        template += u'{{% block content %}}\n{}\n{{% endblock %}}\n'.format(body)
+        template += u'{{% block {} %}}\n{}\n{{% endblock %}}\n'.format(contentblock, body)
         if not template_uri and template_title:
             template_uri = template_date.strftime('%Y-%m-%d_') if template_date else u''
             for c in unicodedata.normalize('NFD', force_bytes(template_title).decode('utf-8')):
